/**
  ******************************************************************************
  * @file     USART1_user.c
  * @author   sinomcu.com-AE
  * @brief    USART1 using
  *    
  *          This file provides functions to USART1 using:
  *      USART_IDIE_Callback();
  *      DMA1_ReceiveComplete_Callback();
  *      DMA1_TransferError_Callback();  
  *      Send_buff_Uart(uint8_t *uart_buff);   
  *      dma_uart_rx_init();
  *
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#include "USART1_user.h"

uint8_t g_uart_recv_buff[UART_BUFF_LEN]={0};
FlagStatus UART_IDIE_Flag = RESET,DMA_RecvComplete_Flag = RESET,DMA_SendComplete_Flag = RESET,DMA_TransferError_Flag = RESET;

/**
  * @brief  Function called by USART1 IRQ Handler when USART IDIE 
  * @param  None
  * @retval None
  */
void USART_IDIE_Callback(void) {
  UART_IDIE_Flag = SET;
}

/**
  * @brief  Function called by DMA1 IRQ Handler when USART Rx is completed 
  * @param  None
  * @retval None
  */
void DMA1_RecvComplete_Callback(void) {
  DMA_RecvComplete_Flag = SET;
}

/**
  * @brief  Function called by DMA1 IRQ Handler when USART Tx is completed 
  * @param  None
  * @retval None
  */
void DMA1_SendComplete_Callback(void) {
  DMA_SendComplete_Flag = SET;
}

/**
  * @brief  Function called in case of error detected in DMA1 IRQ Handler
  * @param  None
  * @retval None
  */
void DMA1_TransferError_Callback(void) {
  DMA_TransferError_Flag = SET;
}

/**
* @brief  DMA Initialization Function
          DMA channel3 receive USART1 data.
* @param  None
* @retval None
*/
void dma_uart_init(void) {
  MS32_DMA_InitTypeDef  DmaInitStruct;

  // config DMA channel 2,USART send
  MS32_DMA_StructInit(&DmaInitStruct);
  DmaInitStruct.PeriphOrM2MSrcAddress = MS32_USART_DMA_GetRegAddr(USART1, MS32_USART_DMA_REG_DATA_TRANSMIT); // USART_TDR address
  DmaInitStruct.MemoryOrM2MDstAddress = (uint32_t)g_uart_recv_buff;
  DmaInitStruct.Direction = MS32_DMA_DIRECTION_MEMORY_TO_PERIPH;
  DmaInitStruct.NbData =  UART_BUFF_LEN;
  DmaInitStruct.PeriphOrM2MSrcIncMode = MS32_DMA_PERIPH_NOINCREMENT;
  DmaInitStruct.MemoryOrM2MDstIncMode = MS32_DMA_MEMORY_INCREMENT;
  DmaInitStruct.PeriphOrM2MSrcDataSize = MS32_DMA_PDATAALIGN_BYTE;
  DmaInitStruct.MemoryOrM2MDstDataSize = MS32_DMA_MDATAALIGN_BYTE;
  DmaInitStruct.Mode = MS32_DMA_MODE_NORMAL;               // next time manual start
  DmaInitStruct.Priority = MS32_DMA_PRIORITY_HIGH;
  MS32_DMA_Init(DMA1,MS32_DMA_CHANNEL_2,&DmaInitStruct);   // this function had enable DMA clock  
  
  // config DMA channel 3,USART recv
  MS32_DMA_StructInit(&DmaInitStruct);
  DmaInitStruct.PeriphOrM2MSrcAddress = MS32_USART_DMA_GetRegAddr(USART1, MS32_USART_DMA_REG_DATA_RECEIVE);  // USART_RDR address
  DmaInitStruct.MemoryOrM2MDstAddress = (uint32_t)g_uart_recv_buff;
  DmaInitStruct.Direction = MS32_DMA_DIRECTION_PERIPH_TO_MEMORY;
  DmaInitStruct.NbData = UART_BUFF_LEN;
  DmaInitStruct.PeriphOrM2MSrcIncMode = MS32_DMA_PERIPH_NOINCREMENT;
  DmaInitStruct.MemoryOrM2MDstIncMode = MS32_DMA_MEMORY_INCREMENT;
  DmaInitStruct.PeriphOrM2MSrcDataSize = MS32_DMA_PDATAALIGN_BYTE;
  DmaInitStruct.MemoryOrM2MDstDataSize = MS32_DMA_MDATAALIGN_BYTE;
  DmaInitStruct.Mode = MS32_DMA_MODE_CIRCULAR;
  DmaInitStruct.Priority = MS32_DMA_PRIORITY_HIGH;
  MS32_DMA_Init(DMA1,MS32_DMA_CHANNEL_3,&DmaInitStruct);   // this function had enable DMA clock

  // enable DMA channel 2 TCIE,TEIE
  MS32_DMA_ITConfig(DMA1,MS32_DMA_CHANNEL_2,MS32_DMA_CCR_TCIE|MS32_DMA_CCR_TEIE,0); 
  MS32_DMA_ClearFlag_TC2(DMA1);  
  // enable DMA channel 3,recv
  MS32_DMA_EnableChannel(DMA1, MS32_DMA_CHANNEL_3);
  // enable DMA channel 3 TCIE,TEIE
  MS32_DMA_ITConfig(DMA1,MS32_DMA_CHANNEL_3,MS32_DMA_CCR_TCIE|MS32_DMA_CCR_TEIE,0);
  MS32_DMA_ClearFlag_TC3(DMA1);
}

/******************************** END OF FILE *********************************/
